import sys
import unittest

import testlib
import resourced_pkg
import resourced_pkg.sub_pkg

@unittest.skipIf(sys.version_info < (3, 7), 'importlib.resources, Python >= 3.7')
class ResourceReaderBaselineTest(testlib.TestCase):
    'Assert out-the-box stdlib behaviours to cross validate remote tests.'
    def test_is_resource(self):
        import importlib.resources

        self.assertFalse(importlib.resources.is_resource(resourced_pkg, 'does_not_exist'))
        self.assertFalse(importlib.resources.is_resource(resourced_pkg, 'sub_dir'))
        self.assertFalse(importlib.resources.is_resource(resourced_pkg.sub_pkg, 'does_not_exist'))

        self.assertTrue(importlib.resources.is_resource(resourced_pkg, 'binary'))
        self.assertTrue(importlib.resources.is_resource(resourced_pkg, 'empty'))
        self.assertTrue(importlib.resources.is_resource(resourced_pkg, 'text.txt'))
        self.assertTrue(importlib.resources.is_resource(resourced_pkg, 'sub_dir/empty'))
        self.assertTrue(importlib.resources.is_resource(resourced_pkg.sub_pkg, 'text.txt'))


@unittest.skipIf(sys.version_info < (3, 7), 'importlib.resources, Python >= 3.7')
class ResourceReaderTest(testlib.RouterMixin, testlib.TestCase):
    def call_is_resource(self, conn):
        import importlib.resources

        self.assertFalse(conn.call(importlib.resources.is_resource, 'resourced_pkg', 'does_not_exist'))
        self.assertFalse(conn.call(importlib.resources.is_resource, 'resourced_pkg', 'sub_dir'))
        self.assertFalse(conn.call(importlib.resources.is_resource, 'resourced_pkg.sub_pkg', 'does_not_exist'))

        self.assertTrue(conn.call(importlib.resources.is_resource, 'resourced_pkg', 'binary'))
        self.assertTrue(conn.call(importlib.resources.is_resource, 'resourced_pkg', 'empty'))
        self.assertTrue(conn.call(importlib.resources.is_resource, 'resourced_pkg', 'text.txt'))
        self.assertTrue(conn.call(importlib.resources.is_resource, 'resourced_pkg', 'sub_dir/empty'))
        self.assertTrue(conn.call(importlib.resources.is_resource, 'resourced_pkg.sub_pkg', 'text.txt'))

    def test_is_resource(self):
        # Uses the same version of Python so we can be sure importlib.resources is present
        # TODO Cross Python version tests
        connection = self.router.local(python_path=sys.executable)
        self.call_is_resource(conn=connection)

    def test_is_resource_2_hops(self):
        # Uses the same version of Python so we can be sure importlib.resources is present
        # TODO Cross Python version tests
        hop_one = self.router.local(python_path=sys.executable)
        hop_two = self.router.local(python_path=sys.executable, via=hop_one)
        self.call_is_resource(conn=hop_two)



import unittest

import testlib

import stdio_checks


class StdIOMixin(testlib.RouterMixin):
    """
    Test that stdin, stdout, and stderr conform to common expectations,
    such as blocking IO.
    """
    def check_can_write_stdout_1_mib(self, context):
        """
        Writing to stdout should not raise EAGAIN. Regression test for
        https://github.com/mitogen-hq/mitogen/issues/712.
        """
        size = 1 * 2**20
        nwritten = context.call(stdio_checks.shout_stdout, size)
        self.assertEqual(nwritten, size)

    def check_stdio_is_blocking(self, context):
        stdin_blocking, stdout_blocking, stderr_blocking = context.call(
            stdio_checks.stdio_is_blocking,
        )
        self.assertTrue(stdin_blocking)
        self.assertTrue(stdout_blocking)
        self.assertTrue(stderr_blocking)


class LocalTest(StdIOMixin, testlib.TestCase):
    def test_can_write_stdout_1_mib(self):
        self.check_can_write_stdout_1_mib(self.router.local())

    def test_stdio_is_blocking(self):
        self.check_stdio_is_blocking(self.router.local())


class SudoTest(StdIOMixin, testlib.TestCase):
    @unittest.skipIf(not testlib.have_sudo_nopassword(), 'Needs passwordless sudo')
    def test_can_write_stdout_1_mib(self):
        self.check_can_write_stdout_1_mib(self.router.sudo())

    @unittest.skipIf(not testlib.have_sudo_nopassword(), 'Needs passwordless sudo')
    def test_stdio_is_blocking(self):
        self.check_stdio_is_blocking(self.router.sudo())



try:
    import in_try
    from in_try import x as y, z
except ImportError:
    import in_except_importerror
    from in_except_importerror import x as y, z
except Exception:
    import in_except_exception
    from in_except_exception import x as y, z



# pyright: reportMissingImports=false
# ruff: noqa: E401 E702 F401 F403

from __future__ import absolute_import

import a
import a.b
import c as d
import e, e.f as g \
    , h; import i

from j import k, l, m as n
from o import *



#!/usr/bin/env python
# Run the Mitogen tests.

import os
import subprocess

import ci_lib

os.environ.update({
    'MITOGEN_LOG_LEVEL': 'debug',
    'SKIP_ANSIBLE': '1',
})

if not ci_lib.have_docker():
    os.environ['SKIP_DOCKER_TESTS'] = '1'

subprocess.check_call(
    "umask 0022; sudo cp '%s' '%s'"
    % (ci_lib.SUDOERS_DEFAULTS_SRC, ci_lib.SUDOERS_DEFAULTS_DEST),
    shell=True,
)
subprocess.check_call(['sudo', 'visudo', '-cf', ci_lib.SUDOERS_DEFAULTS_DEST])
subprocess.check_call(['sudo', '-l'])

interesting = ci_lib.get_interesting_procs()
ci_lib.run('./run_tests -v')
ci_lib.check_stray_processes(interesting)



import os

import mitogen.core

import testlib

class PipeTest(testlib.TestCase):
    def test_pipe_blocking_unspecified(self):
        "Test that unspecified blocking arg (None) behaves same as os.pipe()"
        os_rfd, os_wfd = os.pipe()
        mi_rfp, mi_wfp = mitogen.core.pipe()

        self.assertEqual(mitogen.core.get_blocking(os_rfd),
                         mitogen.core.get_blocking(mi_rfp.fileno()))
        self.assertEqual(mitogen.core.get_blocking(os_wfd),
                         mitogen.core.get_blocking(mi_wfp.fileno()))
        mi_rfp.close()
        mi_wfp.close()
        os.close(os_rfd)
        os.close(os_wfd)

    def test_pipe_blocking_true(self):
        mi_rfp, mi_wfp = mitogen.core.pipe(blocking=True)
        self.assertTrue(mitogen.core.get_blocking(mi_rfp.fileno()))
        self.assertTrue(mitogen.core.get_blocking(mi_wfp.fileno()))
        mi_rfp.close()
        mi_wfp.close()

    def test_pipe_blocking_false(self):
        mi_rfp, mi_wfp = mitogen.core.pipe(blocking=False)
        self.assertFalse(mitogen.core.get_blocking(mi_rfp.fileno()))
        self.assertFalse(mitogen.core.get_blocking(mi_wfp.fileno()))
        mi_rfp.close()
        mi_wfp.close()




import unittest

import ansible_mitogen.utils


class AnsibleVersionTest(unittest.TestCase):
    def test_ansible_version(self):
        self.assertIsInstance(ansible_mitogen.utils.ansible_version, tuple)
        self.assertIsInstance(ansible_mitogen.utils.ansible_version[0], int)
        self.assertIsInstance(ansible_mitogen.utils.ansible_version[1], int)
        self.assertEqual(2, ansible_mitogen.utils.ansible_version[0])



"""
Measure latency of .fork() setup/teardown.
"""

import mitogen
import mitogen.core


@mitogen.main()
def main(router):
    import optparse
    parser = optparse.OptionParser(description=__doc__)
    parser.add_option(
        '-i', '--iterations', type=int, metavar='N', default=200,
        help='Number of iterations (default %default)')
    parser.add_option('--debug', action='store_true')
    opts, args = parser.parse_args()

    t0 = mitogen.core.now()
    for x in mitogen.core.range(opts.iterations):
        t = mitogen.core.now()
        ctx = router.fork(debug=opts.debug)
        ctx.shutdown(wait=True)

    t1 = mitogen.core.now()
    mean = (t1 - t0) / opts.iterations
    print('++ iterations %d, mean %.03f ms' % (opts.iterations, 1e3 * mean))



#!/usr/bin/env python
# Run tests/ansible/all.yml under Ansible and Ansible-Mitogen

from __future__ import print_function

import os
import subprocess
import sys

import ci_lib


with ci_lib.Fold('unit_tests'):
    os.environ['SKIP_MITOGEN'] = '1'
    ci_lib.run('./run_tests -v')


with ci_lib.Fold('job_setup'):
    os.chmod(ci_lib.TESTS_SSH_PRIVATE_KEY_FILE, int('0600', 8))


with ci_lib.Fold('machine_prep'):
    # generate a new ssh key for localhost ssh
    if not os.path.exists(os.path.expanduser("~/.ssh/id_rsa")):
        subprocess.check_call("ssh-keygen -P '' -m pem -f ~/.ssh/id_rsa", shell=True)
        subprocess.check_call("cat ~/.ssh/id_rsa.pub >> ~/.ssh/authorized_keys", shell=True)
        os.chmod(os.path.expanduser('~/.ssh'), int('0700', 8))
        os.chmod(os.path.expanduser('~/.ssh/authorized_keys'), int('0600', 8))

    # also generate it for the sudo user
    if os.system("sudo [ -f ~root/.ssh/id_rsa ]") != 0:
        subprocess.check_call("sudo ssh-keygen -P '' -m pem -f ~root/.ssh/id_rsa", shell=True)
        subprocess.check_call("sudo cat ~root/.ssh/id_rsa.pub | sudo tee -a ~root/.ssh/authorized_keys", shell=True)
        subprocess.check_call('sudo chmod 700 ~root/.ssh', shell=True)
        subprocess.check_call('sudo chmod 600 ~root/.ssh/authorized_keys', shell=True)

    os.chdir(ci_lib.IMAGE_PREP_DIR)
    ci_lib.run("ansible-playbook -c local -i localhost, macos_localhost.yml")

    if os.path.expanduser('~mitogen__user1') == '~mitogen__user1':
        os.chdir(ci_lib.IMAGE_PREP_DIR)
        ci_lib.run("ansible-playbook -c local -i localhost, _user_accounts.yml")

    cmd = ';'.join([
        'from __future__ import print_function',
        'import os, sys',
        'print(sys.executable, os.path.realpath(sys.executable))',
    ])
    for interpreter in ['/usr/bin/python', '/usr/bin/python2', '/usr/bin/python2.7']:
        print(interpreter)
        try:
            subprocess.call([interpreter, '-c', cmd])
        except OSError as exc:
            print(exc)

        print(interpreter, 'with PYTHON_LAUNCHED_FROM_WRAPPER=1')
        environ = os.environ.copy()
        environ['PYTHON_LAUNCHED_FROM_WRAPPER'] = '1'
        try:
            subprocess.call([interpreter, '-c', cmd], env=environ)
        except OSError as exc:
            print(exc)


with ci_lib.Fold('ansible'):
    os.chdir(ci_lib.ANSIBLE_TESTS_DIR)
    playbook = os.environ.get('PLAYBOOK', 'all.yml')
    ci_lib.run('./run_ansible_playbook.py %s %s',
        playbook, ' '.join(sys.argv[1:]))



import fcntl
import os
import sys


def _shout_stdout_py3(size):
    nwritten = sys.stdout.write('A' * size)
    return nwritten


def _shout_stdout_py2(size):
    shout = 'A' * size
    nwritten = 0
    while nwritten < size:
        nwritten += os.write(sys.stdout.fileno(), shout[-nwritten:])
    return nwritten


def shout_stdout(size):
    if sys.version_info > (3, 0):
        return _shout_stdout_py3(size)
    else:
        return _shout_stdout_py2(size)


def file_is_blocking(fobj):
    return not (fcntl.fcntl(fobj.fileno(), fcntl.F_GETFL) & os.O_NONBLOCK)


def stdio_is_blocking():
    return [file_is_blocking(f) for f in [sys.stdin, sys.stdout, sys.stderr]]



import os
import sys
import unittest

import mitogen.imports

import testlib


def testmod_compile(path):
    path = os.path.join(testlib.MODS_DIR, path)
    f = open(path, 'rb')
    co = compile(f.read(), path, 'exec')
    f.close()
    return co


class ScanCodeImportsTest(testlib.TestCase):
    func = staticmethod(mitogen.imports.codeobj_imports)

    @unittest.skipIf(sys.version_info < (3, 0), "Py is 2.x, would be relative")
    def test_default_absolute(self):
        co = testmod_compile('scanning/defaults.py')
        expected = [
            (0, 'a', ()), (0, 'a.b', ()), (0, 'c', ()),
            (0, 'e', ()), (0, 'e.f', ()), (0, 'h', ()),
            (0, 'i', ()),
            (0, 'j', ('k', 'l', 'm')),
            (0, 'o', ('*',)),
        ]
        self.assertEqual(list(self.func(co)), expected)

    @unittest.skipIf(sys.version_info >= (3, 0), "Py is 3.x, would be absolute")
    def test_default_relative(self):
        co = testmod_compile('scanning/defaults.py')
        expected = [
            (-1, 'a', ()), (-1, 'a.b', ()), (-1, 'c', ()),
            (-1, 'e', ()), (-1, 'e.f', ()), (-1, 'h', ()),
            (-1, 'i', ()),
            (-1, 'j', ('k', 'l', 'm')),
            (-1, 'o', ('*',)),
        ]
        self.assertEqual(list(self.func(co)), expected)

    @unittest.skipIf(sys.version_info < (2, 5), "Py is 2.4, no absolute_import")
    def test_explicit_absolute(self):
        co = testmod_compile('scanning/has_absolute_import.py')
        expected = [
            (0, '__future__', ('absolute_import',)),

            (0, 'a', ()), (0, 'a.b', ()), (0, 'c', ()),
            (0, 'e', ()), (0, 'e.f', ()), (0, 'h', ()),
            (0, 'i', ()),
            (0, 'j', ('k', 'l', 'm')),
            (0, 'o', ('*',)),
        ]
        self.assertEqual(list(self.func(co)), expected)

    @unittest.skipIf(sys.version_info < (2, 5), "Py is 2.4, no `from . import x`")
    def test_explicit_relative(self):
        co = testmod_compile('scanning/explicit_relative.py')
        expected = [
            (1, '', ('a',)),
            (1, 'b', ('c', 'd')),
            (3, '', ('f', 'j')),
        ]
        self.assertEqual(list(self.func(co)), expected)

    def test_scoped_class(self):
        # Imports in `class` or `def` are ignored, a bad heuristc to detect
        # lazy imports and skip sending the pre-emptively.
        # See
        # - https://github.com/mitogen-hq/mitogen/issues/682
        # - https://github.com/mitogen-hq/mitogen/issues/1325#issuecomment-3170482014
        co = testmod_compile('scanning/scoped_class.py')
        self.assertEqual(list(self.func(co)), [])

        pass

    def test_scoped_function(self):
        co = testmod_compile('scanning/scoped_function.py')
        self.assertEqual(list(self.func(co)), [])

    @unittest.skipIf(sys.version_info >= (3, 0), "Python is 3.x, which prunes")
    def test_scoped_if_else_unpruned(self):
        co = testmod_compile('scanning/scoped_if_else.py')
        level = (-1, 0)[int(sys.version_info >= (3, 0))]
        expected = [
            (level, 'sys', ()),
            (level, 'in_if_always_true', ()),
            (level, 'in_if_always_true', ('x', 'z')),
            # Python 2.x does no pruning
            (level, 'in_else_never_true', ()),
            (level, 'in_else_never_true', ('x', 'z')),
            (level, 'in_if_py3', ()),
            (level, 'in_if_py3', ('x', 'z')),
            (level, 'in_else_py2', ()),
            (level, 'in_else_py2', ('x', 'z')),
        ]
        self.assertEqual(list(self.func(co)), expected)

    @unittest.skipIf(sys.version_info < (3, 0), "Python is 2.x, which doesn't prune")
    def test_scoped_if_else_pruned(self):
        co = testmod_compile('scanning/scoped_if_else.py')
        level = (-1, 0)[int(sys.version_info >= (3, 0))]
        expected = [
            (level, 'sys', ()),
            (level, 'in_if_always_true', ()),
            (level, 'in_if_always_true', ('x', 'z')),
            # Python 3.x prunes some impossible branches ...
            (level, 'in_if_py3', ()),
            (level, 'in_if_py3', ('x', 'z')),
            # ... but not sys.version_info ones
            (level, 'in_else_py2', ()),
            (level, 'in_else_py2', ('x', 'z')),
        ]
        self.assertEqual(list(self.func(co)), expected)

    def test_scoped_try_except(self):
        co = testmod_compile('scanning/scoped_try_except.py')
        level = (-1, 0)[int(sys.version_info >= (3, 0))]
        expected = [
            (level, 'in_try', ()),
            (level, 'in_try', ('x', 'z')),
            (level, 'in_except_importerror', ()),
            (level, 'in_except_importerror', ('x', 'z')),
            (level, 'in_except_exception', ()),
            (level, 'in_except_exception', ('x', 'z')),
        ]
        self.assertEqual(list(self.func(co)), expected)
